/**
 * An "Emote" as returned by the BTTV API
 */
type BTTV_Emote = {
  id: string;
  code: string;
  imageType: string;
  animated: boolean;
  user: any;
};

/**
 * Query BTTV API for an emote graphic
 *
 * @param code string associated with the emote graphic (often called "emote" itself)
 * @returns `null` iff no emote found or error, else URL to a BTTV emote graphic
 */
export async function bttv_get_url(code: string): Promise<string | null> {
  // search for emotes
  const res = await fetch(
    `https://api.betterttv.net/3/emotes/shared/search?query=${code}&offset=0&limit=50`,
  );
  const content = await res.json();

  // sanity check
  if (!Array.isArray(content)) return null;

  // find exact matches (disregard case)
  const matches = (content as BTTV_Emote[]).filter(
    (be) => be.code.toLowerCase() == code.toLowerCase(),
  );

  // ensure there's emotes left
  if (matches.length <= 0) return null;

  // return first emote's URL
  return `//cdn.betterttv.net/emote/${matches[0].id}/1x.${matches[0].imageType}`;
}
