/**
 * Concurrently create some mapping of input to output elements.
 *
 * @param inputs collection of input elements
 * @param map_fn function mapping inputs to outputs
 * @returns ES6 Map object with inputs mapped to outputs where possible
 */
export async function map_concurrent<In, Out>(
  inputs: Iterable<In>,
  map_fn: (input: In) => Promise<Out | null>,
): Promise<Map<In, Out | null>> {
  // apply map_fn to inputs
  const outputs = await Promise.all([...inputs].map((input) => map_fn(input)));

  // create map object
  return new Map<In, Out | null>(
    [...inputs].map((input, index) => [input, outputs[index]]),
  );
}
